// MIS3502 - Web Service Template
// Created by: Jeremy Shafer
// Fall 2024

// REMINDER - Don't forget to change your database connection
// timeout from 3 seconds to 3 minutes.
// Look under Configuration / General Configuration

// declarations (not all are needed) *****************************************
import qs from 'qs'; //for parsing URL encoded data
import axios from 'axios'; // for calling another API
import mysql from 'mysql2/promise';  //for talking to a database
import { SESClient, SendEmailCommand } from "@aws-sdk/client-ses";


const dboptions = {
  'user' : '',
  'password' : '',
  'database' : '',
  'host' : 'dataanalytics.temple.edu'
};

//global connection variable
var connection;

const features = [

	`Issue a POST against auth and provide a key of username 
	and a key of password. The result will be a JSON object 
	representing the user.`,

	`Issue a GET against clients.  The response will be a JSON objecy
	representing all the active clients.`,

	`UNDER CONSTRUCTION Issue a POST against additem and provide the keys of name, 
	description, and estimatedvalue. The result will be a JSON object 
	representing the newly created item. The keys name and description 
	are strings. The key estimatedvale is a dollar amount.`,

	`UNDER CONSTRUCTION Issue a GET against items and a key of available to get a 
	JSON listing of all available items. The key available is
	either Y or N. The results are ordered by name.`,

	`UNDER CONSTRUCTION Issue a GET against item and provide a key of itemcode. 
	 This will return all the properties of one key.`,

	`UNDER CONSTRUCTION Issue a DELETE against item and provide a key of itemcode, and a key of userid. 
	This will permanently remove the item from the database. 
	Use the userid to verify that the user is an admin.  
	Only admin users may delete items.`,

	"Created by Jeremy Shafer",	
	"Last modified by NoName McHiggens",	
	];


// supporting functions ******* STUDENT MAY EDIT ***********

let clientList = async (res,query) => {
	//work and return the result
	let txtSQL = `SELECT * FROM clients 
				WHERE active = 'Y' 
				ORDER BY lastname, firstname `;
	let [result] = await connection.execute(txtSQL);
	return formatres(res,result,200);
};


let auth = async (res,body) => {
	
	// 1 get the data you need
	let username = body.username;
	let password = body.password;
	
	// 2 error trap
	if (username == undefined){
		return formatres(res,"The key username was undefined",400);
	}
	if (password == undefined){
		return formatres(res,"The key password was undefined",400);
	}
	if (username == ''){
		return formatres(res,"The key username was blank",400);
	}
	if (password == ''){
		return formatres(res,"The key password was blank",400);
	}
	
	// 3 work and return the result
	let txtSQL = `SELECT userid, username, firstname, 
	lastname, admin FROM users 
	WHERE username = ? AND password = ? `;
	
	let [result] = await connection.execute(txtSQL, [username, password]);
	return formatres(res,result,200);
};

let theDatetimeFunction = async (res,query) => {
	//work and return the result
	let [result] = await connection.execute("select DATE_FORMAT(NOW(), '%m-%d-%Y %h:%i %p') AS the_date_and_time");
	return formatres(res,result[0]['the_date_and_time'],200);
}

// do not delete this handy little supporting function
let formatres = async (res, output, statusCode) => {
	
	// kill the global database connection
	if (connection != undefined &&  
		typeof(connection)=='object' &&  
		typeof(connection.end())=='object'  ){
		await connection.end();
	}

	res.statusCode = statusCode;
	res.body = JSON.stringify(output);
	return res;	
};

// do not delete this handy little supportng function
function isEmpty(obj) {
    return Object.keys(obj).length === 0;
}

// My Routing Function ****** STUDENT MAY EDIT **********

let myRoutingFunction = (res,method,path,query,body) => {

	// conditional statements go here.
	// look at the path and method and return the output from the 
	// correct supporting function.

	// Simple GET request with no features specified results
	// in a list of features / instructions
	if (method == "GET" && path == ""){
		return formatres(res, features, 200);
	}

	if (method == "POST" && path == "auth"){
		return auth(res,body);  
	}
	
	if (method == "GET" && path == "clients"){
		return clientList(res,query);  
	}

	return(res);
};


// event handler **** DO NOT EDIT ***********

// Students should not have to change the code here.
// Students should be able to read and understand the code here.

export const handler = async (request) => {

	connection = await mysql.createConnection(dboptions);	

	// identify the method (it will be a string)
	let method = request["httpMethod"];
	
	// identify the path (it will also be a string)
	let fullpath = request["path"];
	
	// we clean the full path up a little bit
	if (fullpath == undefined || fullpath == null){ fullpath = ""};
	let pathitems = fullpath.split("/");
	let path = pathitems[2];
	if (path == undefined || path == null){ path = ""};
	
	// identify the querystring ( we will convert it to 
	//   a JSON object named query)
	let query = request["queryStringParameters"];
	if (query == undefined || query == null){ query={} };
	
	// identify the body (we will convert it to 
	//   a JSON object named body)
	let body = qs.parse(request["body"]);
	if (body == undefined || body == null){ body={} };

	// Create the default response object that will include 
	// the status code, the headers needed by CORS, and
	// the string to be returned formatted as a JSON data structure.
    let res = {
        'statusCode': 400,
        'headers': {
            'Access-Control-Allow-Origin': '*',
            'Access-Control-Allow-Credentials': true  
        },
        'body': JSON.stringify("Feature not found."),
    };

	// run all the parameters through my routing function
	// and return the result
    return myRoutingFunction(res,method,path,query,body);
    
    //*** this is a good place to test one supporting function at a time
    //query={};
    //return theDatetimeFunction(res,query);
    
};